@@ -299,7 +299,7 @@ static void asf_reset_header(AVFormatContext *s) {
 
   for (i=0; i<s->nb_streams; i++) {
     asf_st= s->streams[i]->priv_data;
-    av_free_packet(&asf_st->pkt);
+    av_packet_unref(&asf_st->pkt);
     asf_st->frag_offset=0;
     asf_st->seq=0;
   }
@@ -917,7 +917,7 @@ static void detach_stream(lives_clip_data_t *cdata) {
     av_free(priv->ctx);
   }
 
-  if (priv->picture!=NULL) av_frame_free(&priv->picture);
+  if (priv->picture!=NULL) av_frame_unref(&priv->picture);
 
   priv->ctx=NULL;
   priv->picture=NULL;
@@ -2564,7 +2564,7 @@ static lives_clip_data_t *asf_clone(lives_clip_data_t *cdata) {
 
   }
 
-  if (dpriv->picture!=NULL) av_frame_free(&dpriv->picture);
+  if (dpriv->picture!=NULL) av_frame_unref(&dpriv->picture);
   dpriv->picture=NULL;
 
   return clone;
@@ -2640,7 +2640,7 @@ lives_clip_data_t *get_clip_data(const char *URI, lives_clip_data_t *cdata) {
   cdata->asigned=TRUE;
   cdata->ainterleaf=TRUE;
 
-  if (priv->picture!=NULL) av_frame_free(&priv->picture);
+  if (priv->picture!=NULL) av_frame_unref(&priv->picture);
   priv->picture=NULL;
 
   return cdata;
@@ -2766,7 +2766,7 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
 
   if (tframe!=priv->last_frame) {
 
-    if (priv->picture!=NULL) av_frame_free(&priv->picture);
+    if (priv->picture!=NULL) av_frame_unref(&priv->picture);
     priv->picture=NULL;
 
     if (priv->last_frame==-1 || (tframe<priv->last_frame) || (tframe - priv->last_frame > rescan_limit)) {
@@ -2846,7 +2846,7 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
 
       // otherwise discard this frame
       if (got_picture) {
-        av_frame_free(&priv->picture);
+        av_frame_unref(&priv->picture);
         priv->picture=NULL;
         tfrag=-1;
         nextframe++;
