@@ -1,5 +1,5 @@
 /// LiVES - avformat plugin
-// (c) G. Finch 2010 <salsaman@xs4all.nl,salsaman@gmail.com>
+// (c) G. Finch 2010 - 2016 <salsaman+lives@gmail.com>
 // released under the GNU GPL 3 or later
 // see file COPYING or www.gnu.org for details
 //
@@ -81,7 +81,7 @@ static int stream_peek(int fd, unsigned char *str, size_t len) {
 
 
 void get_samps_and_signed(enum AVSampleFormat sfmt, int *asamps, boolean *asigned) {
-  *asamps=av_get_bytes_per_sample(sfmt)*8;
+  *asamps=av_get_bits_per_sample(sfmt);
 
   switch (sfmt) {
   case AV_SAMPLE_FMT_U8:
@@ -187,7 +187,7 @@ static int64_t get_real_last_frame(lives_clip_data_t *cdata, boolean allow_longe
 static boolean attach_stream(lives_clip_data_t *cdata, boolean isclone) {
   // open the file and get a handle
   lives_av_priv_t *priv=cdata->priv;
-  AVProbeData   pd;
+  AVProbeData pd={0};
   AVInputFormat *fmt;
 
   AVCodec *vdecoder;
@@ -219,8 +219,8 @@ static boolean attach_stream(lives_clip_data_t *cdata, boolean isclone) {
   pd.filename=cdata->URI;
 
   pd.buf=calloc(128,32);
-
-  if ((pd.buf_size=stream_peek(priv->fd,pd.buf,2261))<2261) {
+  
+  if ((pd.buf_size=stream_peek(priv->fd,pd.buf,AVPROBE_PADDING_SIZE))<AVPROBE_PADDING_SIZE) {
     fprintf(stderr, "couldn't peek stream %d\n",pd.buf_size);
     return FALSE;
   }
@@ -264,12 +264,19 @@ skip_probe:
   priv->last_frame=-1;
   priv->black_fill=FALSE;
 
+  priv->needs_packet=TRUE;
+
   /* Open it */
   if (avformat_open_input(&priv->ic, cdata->URI, priv->fmt, NULL)) {
     fprintf(stderr, "avformat_open_input failed\n");
     return FALSE;
   }
 
+  if (priv->ic->pb==NULL) {
+    fprintf(stderr, "avformat stream not video\n");
+    return FALSE;
+  }
+
   if (!priv->ic->pb->seekable) {
     fprintf(stderr, "avformat stream non-seekable\n");
     return FALSE;
@@ -318,8 +325,25 @@ skip_probe:
 
 skip_init:
     s = priv->ic->streams[i];
-    cc = s->codec;
 
+
+#ifdef API_3_1
+    cc = avcodec_alloc_context3(NULL);
+    if (!cc) {
+      fprintf(stderr,"avcodec_decoder: out of memory\n");
+      return FALSE;
+    }
+ 
+    ret = avcodec_parameters_to_context(cc, s->codecpar);
+    if (ret < 0) {
+      fprintf(stderr,"avcodec_decoder: avparms to context failed\n");
+      return FALSE;
+    }
+
+#else
+ cc = s->codec;
+#endif
+    
     // vlc_fourcc_t fcc;
     //const char *psz_type = "unknown";
 
@@ -379,6 +403,11 @@ skip_init:
       cdata->palettes[0]=avi_pix_fmt_to_weed_palette(cc->pix_fmt,&cdata->YUV_clamping);
       cdata->palettes[1]=WEED_PALETTE_END;
 
+      if (cdata->palettes[0]==WEED_PALETTE_END) {
+	fprintf(stderr,"avcodec_decoder: no usable palette found (%d)\n",cc->pix_fmt);
+	return FALSE;
+      }
+
       sprintf(cdata->video_name,"%s",cc->codec_name);
 
       cdata->par=cc->sample_aspect_ratio.num/cc->sample_aspect_ratio.den;
@@ -579,7 +608,7 @@ skip_init:
   }
 
   if (priv->vstream==-1) {
-    fprintf(stderr,"avcodec_decoder: no video stream found");
+    fprintf(stderr,"avcodec_decoder: no video stream found\n");
     return FALSE;
   }
 
@@ -630,18 +659,23 @@ skip_init:
 static void detach_stream(lives_clip_data_t *cdata) {
   // close the file, free the decoder
   lives_av_priv_t *priv=cdata->priv;
-  close(priv->fd);
+  if (priv->fd>-1) close(priv->fd);
 
   // will close and free the context
-  if (priv->ic !=NULL) av_close_input_file(priv->ic);
-
+  if (priv->ic !=NULL) {
+    avformat_close_input(&priv->ic);
+  }
+  
   if (cdata->palettes!=NULL) free(cdata->palettes);
   cdata->palettes=NULL;
 
   priv->ctx=NULL;
 
+  if (!priv->needs_packet)
+    av_packet_unref(&priv->packet);
+
   if (priv->pFrame!=NULL) {
-    av_frame_free(&priv->pFrame);
+    av_frame_unref(&priv->pFrame);
     priv->pFrame=NULL;
   }
 
@@ -819,6 +853,7 @@ lives_clip_data_t *get_clip_data(const char *URI, lives_clip_data_t *cdata) {
     }
     cdata->URI=strdup(URI);
     if (!attach_stream(cdata,FALSE)) {
+      detach_stream(cdata);
       free(cdata->URI);
       cdata->URI=NULL;
       clip_data_free(cdata);
@@ -848,6 +883,8 @@ rescan:
   }
 
   if (real_frames<=0) {
+    fprintf(stderr,
+            "avformat_decoder: ERROR - could not find the last frame\navformat_decoder: I will pass on this file as it may be broken.\n");
     detach_stream(cdata);
     free(cdata->URI);
     cdata->URI=NULL;
@@ -863,7 +900,7 @@ rescan:
 
   cdata->nframes=real_frames;
 
-  if (priv->pFrame!=NULL) av_frame_free(&priv->pFrame);
+  if (priv->pFrame!=NULL) av_frame_unref(&priv->pFrame);
   priv->pFrame=NULL;
 
   return cdata;
@@ -935,7 +972,7 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
   double time;
 
   AVStream *s = priv->ic->streams[priv->vstream];
-  AVCodecContext *cc = s->codec;
+  AVCodecContext *cc;
 
   int64_t target_pts, MyPts, seek_target=10000000000;
   int64_t timex;
@@ -946,16 +983,39 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
   boolean hit_target=FALSE;
 
   int gotFrame;
-  int p,i;
+
   int xheight=cdata->frame_height,pal=cdata->current_palette,nplanes=1,dstwidth=cdata->width,psize=1;
   int btop=cdata->offs_y,bbot=xheight-1-btop;
   int bleft=cdata->offs_x,bright=cdata->frame_width-cdata->width-bleft;
   int y_black=(cdata->YUV_clamping==WEED_YUV_CLAMPING_CLAMPED)?16:0;
+  int ret;
 
   int jump_frames;
 
+  register int p,i;
+
   if (tframe<0||tframe>=cdata->nframes||cdata->fps==0.) return FALSE;
 
+
+
+#ifdef API_3_1
+  cc = avcodec_alloc_context3(NULL);
+  if (!cc) {
+    fprintf(stderr,"avcodec_decoder: out of memory\n");
+    return FALSE;
+  }
+ 
+  ret = avcodec_parameters_to_context(cc, s->codecpar);
+  if (ret < 0) {
+    fprintf(stderr,"avcodec_decoder: avparms to context failed\n");
+    return FALSE;
+  }
+
+#else
+  cc = s->codec;
+#endif
+
+  
   //cc->get_buffer = our_get_buffer;
   //cc->release_buffer = our_release_buffer;
 
@@ -1003,11 +1063,12 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
   if (priv->pFrame==NULL||tframe!=priv->last_frame) {
     // same frame -> we reuse priv-pFrame;
 
+    //#define DEBUG
 #ifdef DEBUG
     fprintf(stderr,"pt a1 %d %ld\n",priv->last_frame,tframe);
 #endif
 
-    if (priv->pFrame!=NULL) av_frame_free(&priv->pFrame);
+    if (priv->pFrame!=NULL) av_frame_unref(&priv->pFrame);
     priv->pFrame=NULL;
 
     time=(double)tframe/cdata->fps;
@@ -1030,33 +1091,39 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
 
       seek_target=av_rescale_q(xtarget_pts, AV_TIME_BASE_Q, s->time_base);
       av_seek_frame(priv->ic, priv->vstream, seek_target, AVSEEK_FLAG_BACKWARD);
+#ifdef DEBUG
+    fprintf(stderr,"pt a2 %d %ld\n",priv->last_frame,seek_target);
+#endif
       avcodec_flush_buffers(cc);
       priv->black_fill=FALSE;
       MyPts=-1;
+      priv->needs_packet=TRUE;
     } else {
       MyPts=(priv->last_frame+1.)/cdata->fps*(double)AV_TIME_BASE;
     }
 
     //
 
-    while (1) {
-      do {
-        int ret;
+    do {
+      if (priv->needs_packet) {
+        do {
 
-        ret=av_read_frame(priv->ic, &priv->packet);
+          ret=av_read_frame(priv->ic, &priv->packet);
 
 #ifdef DEBUG
-        fprintf(stderr,"ret was %d for tframe %ld\n",ret,tframe);
+          fprintf(stderr,"ret was %d for tframe %ld\n",ret,tframe);
 #endif
-        if (ret<0) {
-          av_free_packet(&priv->packet);
-          priv->last_frame=tframe;
-          if (pixel_data==NULL) return FALSE;
-          priv->black_fill=TRUE;
-          goto framedone;
-        }
+          if (ret<0) {
+            priv->needs_packet=TRUE;
+            priv->last_frame=tframe;
+            if (pixel_data==NULL) return FALSE;
+            priv->black_fill=TRUE;
+            goto framedone;
+          }
 
-      } while (priv->packet.stream_index!=priv->vstream);
+        } while (priv->packet.stream_index!=priv->vstream);
+
+      }
 
       if (MyPts==-1) {
         MyPts = priv->packet.pts;
@@ -1070,18 +1137,24 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
       // decode any frames from this packet
       if (priv->pFrame==NULL) priv->pFrame=av_frame_alloc();
 
-
 #if LIBAVCODEC_VERSION_MAJOR >= 52
-      avcodec_decode_video2(cc, priv->pFrame, &gotFrame, &priv->packet);
+      ret=avcodec_decode_video2(cc, priv->pFrame, &gotFrame, &priv->packet);
+      if (ret<0) return FALSE;
+      ret = FFMIN(ret, priv->packet.size);
+      priv->packet.data+=ret;
+      priv->packet.size-=ret;
 #else
-      avcodec_decode_video(cc, priv->pFrame, &gotFrame, priv->packet.data, priv->packet.size);
+      ret=avcodec_decode_video(cc, priv->pFrame, &gotFrame, priv->packet.data, priv->packet.size);
+      priv->packet.size=0;
 #endif
 
 #ifdef DEBUG
       fprintf(stderr,"pt 1 %ld %d %ld\n",tframe,gotFrame,MyPts);
 #endif
 
-      av_free_packet(&priv->packet);
+      if (priv->packet.size==0) {
+        priv->needs_packet=TRUE;
+      }
 
       if (MyPts >= target_pts - 100) hit_target=TRUE;
 
@@ -1090,7 +1163,7 @@ boolean get_frame(const lives_clip_data_t *cdata, int64_t tframe, int *rowstride
       // otherwise discard this frame
       if (gotFrame) {
         MyPts+=(double)AV_TIME_BASE/cdata->fps;
-        av_frame_free(&priv->pFrame);
+        av_frame_unref(&priv->pFrame);
         priv->pFrame=NULL;
       }
 
