@@ -1,5 +1,5 @@
 // LiVES - mkv decoder plugin
-// (c) G. Finch 2011 <salsaman@xs4all.nl,salsaman@gmail.com>
+// (c) G. Finch 2011 - 2016 <salsaman@gmail.com>
 
 /*
  * This file is free software; you can redistribute it and/or
@@ -40,7 +40,7 @@
 #include <sys/stat.h>
 #include <pthread.h>
 
-const char *plugin_version="LiVES mkv decoder version 1.2";
+const char *plugin_version="LiVES mkv decoder version 1.3";
 
 #ifdef HAVE_AV_CONFIG_H
 #undef HAVE_AV_CONFIG_H
@@ -901,7 +901,7 @@ static int matroska_merge_packets(AVPacket *out, AVPacket *in) {
   out->data = newdata;
   memcpy(out->data+out->size, in->data, in->size);
   out->size += in->size;
-  av_destruct_packet(in);
+  av_packet_unref(in);
   av_free(in);
   return 0;
 }
@@ -1797,7 +1797,7 @@ static void detach_stream(lives_clip_data_t *cdata) {
 
   avcodec_close(priv->vidst->codec);
 
-  if (priv->picture!=NULL) av_frame_free(&priv->picture);
+  if (priv->picture!=NULL) av_frame_unref(&priv->picture);
 
   priv->ctx=NULL;
   priv->codec=NULL;
@@ -2301,7 +2301,7 @@ static lives_clip_data_t *mkv_clone(lives_clip_data_t *cdata) {
     clone->ainterleaf=TRUE;
   }
 
-  if (dpriv->picture!=NULL) av_frame_free(&dpriv->picture);
+  if (dpriv->picture!=NULL) av_frame_unref(&dpriv->picture);
   dpriv->picture=NULL;
 
   dpriv->last_frame=-1;
@@ -2392,7 +2392,7 @@ lives_clip_data_t *get_clip_data(const char *URI, lives_clip_data_t *cdata) {
   cdata->asigned=TRUE;
   cdata->ainterleaf=TRUE;
 
-  if (priv->picture!=NULL) av_frame_free(&priv->picture);
+  if (priv->picture!=NULL) av_frame_unref(&priv->picture);
   priv->picture=NULL;
 
   return cdata;
@@ -2489,7 +2489,7 @@ static void matroska_clear_queue(MatroskaDemuxContext *matroska) {
   if (matroska->packets) {
     int n;
     for (n = 0; n < matroska->num_packets; n++) {
-      av_free_packet(matroska->packets[n]);
+      av_packet_unref(matroska->packets[n]);
       free(matroska->packets[n]);
     }
     av_freep(&matroska->packets);
