@@ -98,7 +98,6 @@ static int getenv_piece(char *target, size_t tlen, char *envvar, int num) {
   if (str1!=NULL) snprintf(target,tlen,"%s",str1);
   else return 0;
   return 1;
-
 }
 
 ////////////////////////////////////////////////////////////////
@@ -276,6 +275,9 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
     int *pal;
     int pnum,wnum,num_weed_params;
     char **rfx_strings=NULL;
+
+    char *blacklist[2]= {"Timeout indicator",NULL};
+
     weed_plant_t **out_chantmpls,**in_chantmpls=NULL;
     weed_plant_t **in_params=NULL,*gui,*pgui;
 
@@ -321,17 +323,23 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
 
     double vald;
     char *valch;
+
     f0r_param_color_t valcol;
     f0r_param_position_t valpos;
+
     int is_unstable;
 
+    int blacklisted;
+
+    register int i;
+
 #ifdef CAN_GET_DEF
     f0r_instance_t f0r_inst;
     f0r_get_param_value_f f0r_get_param_value=NULL;
 #endif
 
     char *fpp=getenv("FREI0R_PATH");
-
+#define DEBUG
 #ifndef DEBUG
     int new_stdout=dup(1);
     int new_stderr=dup(2);
@@ -339,6 +347,7 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
     close(1);
     close(2);
 #endif
+
     if (fpp!=NULL) {
       vdirval=10;
     } else {
@@ -358,7 +367,9 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
         if (curvdir!=NULL) closedir(curvdir);
         curvdir=opendir(vdir2);
         if (curvdir==NULL) vdirval=4;
-        else vdirval=3;
+        else {
+	  vdirval=3;
+	}
       }
 
       if (vdirval==4) {
@@ -371,11 +382,15 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
         vdirval=5;
       }
 
+
       if (vdirval>9) {
-        if (!getenv_piece(vdir1,PATH_MAX,fpp,vdirval-10)) {
+	char *fpp_copy=strdup(fpp);
+        if (!getenv_piece(vdir1,PATH_MAX,fpp_copy,vdirval-10)) {
+	  free(fpp_copy);
           vdirval=6;
           break;
         }
+	free(fpp_copy);
 
         if (!strlen(vdir1)) {
           vdirval++;
@@ -392,38 +407,40 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
       weed_memset(vendor_name,0,1);
 
       do {
+	snprintf(dir1,PATH_MAX,"%s/%s",vdir1,vendor_name);
 
-        snprintf(dir1,PATH_MAX,"%s/%s",vdir1,vendor_name);
-
-        if (vdirval<10) {
-          snprintf(dir2,PATH_MAX,"%s/%s",vdir2,vendor_name);
-          snprintf(dir3,PATH_MAX,"%s/%s",vdir3,vendor_name);
-        }
+	if (vdirval<10) {
+	  snprintf(dir2,PATH_MAX,"%s/%s",vdir2,vendor_name);
+	  snprintf(dir3,PATH_MAX,"%s/%s",vdir3,vendor_name);
+	}
 
         vdirent=readdir(curvdir);
 
-        if (vdirent==NULL) {
-          closedir(curvdir);
-          curvdir=NULL;
-          vdirval++;
-          break;
-        }
-
-        if (!strncmp(vdirent->d_name,"..",strlen(vdirent->d_name))) continue;
-
-        snprintf(vendor_name,PATH_MAX,"%s",vdirent->d_name);
+        if (vdirent!=NULL) {
+	  if (!strncmp(vdirent->d_name,"..",strlen(vdirent->d_name))) continue;
+	  snprintf(vendor_name,PATH_MAX,"%s",vdirent->d_name);
+	}
 
         if (vdirval==1) {
           curdir=opendir(dir3);
-          if (curdir==NULL) continue;
+          if (curdir==NULL) {
+	    if (vdirent==NULL) break;
+	    continue;
+	  }
         } else if (vdirval==3) {
           if (curdir!=NULL) closedir(curdir);
           curdir=opendir(dir2);
-          if (curdir==NULL) continue;
+          if (curdir==NULL) {
+	    if (vdirent==NULL) break;
+	    continue;
+	  }
         } else if (vdirval==5||vdirval>9) {
           if (curdir!=NULL) closedir(curdir);
           curdir=opendir(dir1);
-          if (curdir==NULL) continue;
+          if (curdir==NULL) {
+	    if (vdirent==NULL) break;
+	    continue;
+	  }
         }
 
         finished=0;
@@ -502,7 +519,17 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
           (*f0r_init)();
           (*f0r_get_plugin_info)(&f0rinfo);
 
-          if (f0rinfo.frei0r_version!=FREI0R_MAJOR_VERSION) {
+          blacklisted=0;
+
+          for (i=0; blacklist[i]!=NULL; i++) {
+            if (!strcmp(f0rinfo.name,blacklist[i])) {
+              fprintf(stderr,"Warning, frei0r plugin skipping blacklisted plugin %s\n",f0rinfo.name);
+              blacklisted=1;
+              break;
+            }
+          }
+
+          if (blacklisted||f0rinfo.frei0r_version!=FREI0R_MAJOR_VERSION) {
             (*f0r_deinit)();
             dlclose(handle);
             continue;
@@ -878,10 +905,12 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
         }
         // end vendor dir
       } while (vdirent!=NULL);
+      if (curvdir!=NULL) closedir(curvdir);
+      curvdir=NULL;
+      vdirval++;
       if (curdir!=NULL) closedir(curdir);
     }
     // end frei0r dirs
-    if (curvdir!=NULL) closedir(curvdir);
 
 #ifndef DEBUG
     dup2(new_stdout,1);
