@@ -585,7 +585,9 @@ weed_plant_t *weed_setup(weed_bootstrap_f weed_boot) {
 
     while (1) {
       // split by :
+      char *lpp_copy=strdup(lpp);
       getenv_piece(vdir,PATH_MAX,lpp,vdirval);
+      free(lpp_copy);
       if (!strlen(vdir)||vdir==NULL) break;
       curvdir=opendir(vdir);
       if (curvdir==NULL) break;
