@@ -1,6 +1,6 @@
 // ce_thumbs.c
 // LiVES
-// (c) G. Finch 2013 <salsaman@gmail.com>
+// (c) G. Finch 2013 - 2016 <salsaman@gmail.com>
 // Released under the GNU GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -18,6 +18,7 @@
 #include "effects.h"
 #include "paramwindow.h"
 #include "ce_thumbs.h"
+#include "callbacks.h"
 
 static LiVESWidget **fxcombos;
 static LiVESWidget **pscrolls;
@@ -254,8 +255,7 @@ void start_ce_thumb_mode(void) {
     fxcombos[i]=lives_standard_combo_new(NULL,FALSE,fxlist,LIVES_BOX(hbox),NULL);
 
     if (fxlist!=NULL) {
-      lives_list_free_strings(fxlist);
-      lives_list_free(fxlist);
+      lives_list_free_all(&fxlist);
       lives_combo_set_active_index(LIVES_COMBO(fxcombos[i]),rte_key_getmode(i+1));
     } else {
       lives_widget_set_sensitive(key_checks[i],FALSE);
@@ -502,7 +502,8 @@ void ce_thumbs_add_param_box(int key, boolean remove) {
 
   do {
     weed_instance_ref(ninst);
-  } while (weed_plant_has_leaf(ninst,"host_next_instance")&&(ninst=weed_get_plantptr_value(ninst,"host_next_instance",&error))!=NULL);
+  } while (weed_plant_has_leaf(ninst,WEED_LEAF_HOST_NEXT_INSTANCE)&&
+           (ninst=weed_get_plantptr_value(ninst,WEED_LEAF_HOST_NEXT_INSTANCE,&error))!=NULL);
 
 
   // here we just check if we have any params to display
@@ -545,7 +546,7 @@ void ce_thumbs_add_param_box(int key, boolean remove) {
   on_fx_pre_activate(rfx,1,vbox);
 
   // record the key so we know whose parameters to record later
-  weed_set_int_value((weed_plant_t *)rfx->source,"host_key",key);
+  weed_set_int_value((weed_plant_t *)rfx->source,WEED_LEAF_HOST_KEY,key);
 
   lives_widget_object_set_data(LIVES_WIDGET_OBJECT(pscrolls[key]),"pinned",LIVES_INT_TO_POINTER(FALSE));
   lives_widget_object_set_data(LIVES_WIDGET_OBJECT(pscrolls[key]),"update",LIVES_INT_TO_POINTER(FALSE));
@@ -657,8 +658,7 @@ void ce_thumbs_reset_combo(int key) {
   lives_combo_populate(LIVES_COMBO(fxcombos[key]),fxlist);
   if (fxlist!=NULL) {
     lives_widget_set_sensitive(key_checks[key],TRUE);
-    lives_list_free_strings(fxlist);
-    lives_list_free(fxlist);
+    lives_list_free_all(&fxlist);
     mode=rte_key_getmode(key+1);
     ce_thumbs_set_mode_combo(key,mode);
     if (rte_keymode_get_instance(key+1,mode)!=NULL) ce_thumbs_add_param_box(key,TRUE);
