@@ -1,6 +1,6 @@
 // effects.c
 // LiVES (lives-exe)
-// (c) G. Finch 2003 - 2014
+// (c) G. Finch 2003 - 2016
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -29,6 +29,7 @@
 #include "cvirtual.h"
 #include "resample.h"
 #include "ce_thumbs.h"
+#include "callbacks.h"
 
 
 //////////// Effects ////////////////
@@ -208,7 +209,7 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
     lives_free(tmp);
   }
 
-  if (!mainw->keep_pre) unlink(cfile->info_file);
+  if (!mainw->keep_pre) lives_rm(cfile->info_file);
 
   if (!mainw->internal_messaging&&!mainw->keep_pre) {
     if (cfile->frame_index_back!=NULL) {
@@ -307,9 +308,9 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
       // set out_channel dimensions for resizers / generators
       int error;
       weed_plant_t *first_out=get_enabled_channel((weed_plant_t *)rfx->source,0,FALSE);
-      weed_plant_t *first_ot=weed_get_plantptr_value(first_out,"template",&error);
-      weed_set_int_value(first_out,"width",weed_get_int_value(first_ot,"host_width",&error));
-      weed_set_int_value(first_out,"height",weed_get_int_value(first_ot,"host_height",&error));
+      weed_plant_t *first_ot=weed_get_plantptr_value(first_out,WEED_LEAF_TEMPLATE,&error);
+      weed_set_int_value(first_out,WEED_LEAF_WIDTH,weed_get_int_value(first_ot,WEED_LEAF_HOST_WIDTH,&error));
+      weed_set_int_value(first_out,WEED_LEAF_HEIGHT,weed_get_int_value(first_ot,WEED_LEAF_HOST_HEIGHT,&error));
     }
   }
 
@@ -404,9 +405,9 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
     } else {
       int error;
       weed_plant_t *first_out=get_enabled_channel((weed_plant_t *)rfx->source,0,FALSE);
-      weed_plant_t *first_ot=weed_get_plantptr_value(first_out,"template",&error);
-      cfile->hsize=weed_get_int_value(first_ot,"host_width",&error);
-      cfile->vsize=weed_get_int_value(first_ot,"host_height",&error);
+      weed_plant_t *first_ot=weed_get_plantptr_value(first_out,WEED_LEAF_TEMPLATE,&error);
+      cfile->hsize=weed_get_int_value(first_ot,WEED_LEAF_HOST_WIDTH,&error);
+      cfile->vsize=weed_get_int_value(first_ot,WEED_LEAF_HOST_HEIGHT,&error);
     }
 
     if (rfx->num_in_channels>0) {
@@ -431,7 +432,7 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
     char *com=lives_strdup_printf("%s mv_pre \"%s\" %d %d \"%s\"",prefs->backend_sync, cfile->handle,cfile->progress_start,
                                   cfile->progress_end,get_image_ext_for_type(cfile->img_type));
 
-    unlink(cfile->info_file);
+    lives_rm(cfile->info_file);
     mainw->cancelled=CANCEL_NONE;
     lives_system(com,FALSE);
     lives_free(com);
@@ -493,7 +494,7 @@ boolean do_effect(lives_rfx_t *rfx, boolean is_preview) {
 
       if (!got_no_frames) mainw->current_file=new_file;
     } else {
-      char *tfile=make_image_file_name(cfile,cfile->frames,prefs->image_ext);
+      char *tfile=make_image_file_name(cfile,cfile->frames,get_image_ext_for_type(cfile->img_type));
 
       if (!lives_file_test(tfile, LIVES_FILE_TEST_EXISTS)) {
         get_frame_count(mainw->current_file);
@@ -644,8 +645,8 @@ lives_render_error_t realfx_progress(boolean reset) {
     mainw->rowstride_alignment=mainw->rowstride_alignment_hint;
 
     layer=weed_plant_new(WEED_PLANT_CHANNEL);
-    weed_set_int_value(layer,"clip",mainw->current_file);
-    weed_set_int_value(layer,"frame",i);
+    weed_set_int_value(layer,WEED_LEAF_CLIP,mainw->current_file);
+    weed_set_int_value(layer,WEED_LEAF_FRAME,i);
 
     frameticks=(i-cfile->start+1.)/cfile->fps*U_SECL;
 
@@ -658,7 +659,7 @@ lives_render_error_t realfx_progress(boolean reset) {
     layer=on_rte_apply(layer, 0, 0, (weed_timecode_t)frameticks);
 
     if (!has_video_filters(TRUE)||resize_instance!=NULL) {
-      layer_palette=weed_get_int_value(layer,"current_palette",&weed_error);
+      layer_palette=weed_get_int_value(layer,WEED_LEAF_CURRENT_PALETTE,&weed_error);
 
       if (resize_instance==NULL) resize_layer(layer,cfile->hsize,cfile->vsize,LIVES_INTERP_BEST,layer_palette,0);
 
@@ -797,7 +798,7 @@ boolean on_realfx_activate_inner(int type, lives_rfx_t *rfx) {
       else {
         char *com=lives_strdup_printf("%s undo_audio %s",prefs->backend_sync,cfile->handle);
         mainw->com_failed=FALSE;
-        unlink(cfile->info_file);
+        lives_rm(cfile->info_file);
         lives_system(com,FALSE);
         lives_free(com);
       }
@@ -827,16 +828,12 @@ void on_realfx_activate(LiVESMenuItem *menuitem, livespointer user_data) {
     if (!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_FRAMES)&&(mainw->xlays=
           layout_frame_is_affected(mainw->current_file,1))!=NULL) {
       if (!do_layout_alter_frames_warning()) {
-        lives_list_free_strings(mainw->xlays);
-        lives_list_free(mainw->xlays);
-        mainw->xlays=NULL;
+        lives_list_free_all(&mainw->xlays);
         return;
       }
       add_lmap_error(LMAP_ERROR_ALTER_FRAMES,cfile->name,(livespointer)cfile->layout_map,mainw->current_file,0,0.,
                      cfile->stored_layout_frame>0);
-      lives_list_free_strings(mainw->xlays);
-      lives_list_free(mainw->xlays);
-      mainw->xlays=NULL;
+      lives_list_free_all(&mainw->xlays);
       has_lmap_error=TRUE;
     }
 
@@ -844,17 +841,13 @@ void on_realfx_activate(LiVESMenuItem *menuitem, livespointer user_data) {
     if (!(prefs->warning_mask&WARN_MASK_LAYOUT_ALTER_AUDIO)&&
         (mainw->xlays=layout_audio_is_affected(mainw->current_file,0.))!=NULL) {
       if (!do_layout_alter_audio_warning()) {
-        lives_list_free_strings(mainw->xlays);
-        lives_list_free(mainw->xlays);
-        mainw->xlays=NULL;
+        lives_list_free_all(&mainw->xlays);
         return;
       }
       add_lmap_error(LMAP_ERROR_ALTER_AUDIO,cfile->name,(livespointer)cfile->layout_map,mainw->current_file,0,0.,
                      cfile->stored_layout_audio>0.);
       has_lmap_error=TRUE;
-      lives_list_free_strings(mainw->xlays);
-      lives_list_free(mainw->xlays);
-      mainw->xlays=NULL;
+      lives_list_free_all(&mainw->xlays);
     }
   }
 
@@ -898,8 +891,8 @@ weed_plant_t *on_rte_apply(weed_plant_t *layer, int opwidth, int opheight, weed_
   if (resize_instance!=NULL) {
     lives_filter_error_t filter_error;
     weed_plant_t *init_event=weed_plant_new(WEED_PLANT_EVENT);
-    weed_set_int_value(init_event,"in_tracks",0);
-    weed_set_int_value(init_event,"out_tracks",0);
+    weed_set_int_value(init_event,WEED_LEAF_IN_TRACKS,0);
+    weed_set_int_value(init_event,WEED_LEAF_OUT_TRACKS,0);
 
     filter_error=weed_apply_instance(resize_instance,init_event,layers,0,0,tc);
     filter_error=filter_error; // stop compiler complaining
@@ -945,14 +938,15 @@ void deinterlace_frame(weed_plant_t *layer, weed_timecode_t tc) {
   layers[0]=layer;
 
   init_event=weed_plant_new(WEED_PLANT_EVENT);
-  weed_set_int_value(init_event,"in_tracks",0);
-  weed_set_int_value(init_event,"out_tracks",0);
+  weed_set_int_value(init_event,WEED_LEAF_IN_TRACKS,0);
+  weed_set_int_value(init_event,WEED_LEAF_OUT_TRACKS,0);
 
 deint1:
 
   weed_apply_instance(deint_instance,init_event,layers,0,0,tc);
 
-  if (weed_plant_has_leaf(deint_instance,"host_next_instance")) next_inst=weed_get_plantptr_value(deint_instance,"host_next_instance",&error);
+  if (weed_plant_has_leaf(deint_instance,WEED_LEAF_HOST_NEXT_INSTANCE)) next_inst=weed_get_plantptr_value(deint_instance,
+        WEED_LEAF_HOST_NEXT_INSTANCE,&error);
   else next_inst=NULL;
 
   weed_call_deinit_func(deint_instance);
@@ -993,13 +987,14 @@ weed_plant_t *get_blend_layer(weed_timecode_t tc) {
 
   blend_file->last_frameno=blend_file->frameno;
 
-  blend_file->frameno=calc_new_playback_position(mainw->blend_file,blend_tc,(uint64_t *)&ntc);
+  if (!cfile->play_paused)
+    blend_file->frameno=calc_new_playback_position(mainw->blend_file,blend_tc,(uint64_t *)&ntc);
 
   blend_tc=ntc;
 
   mainw->blend_layer=weed_plant_new(WEED_PLANT_CHANNEL);
-  weed_set_int_value(mainw->blend_layer,"clip",mainw->blend_file);
-  weed_set_int_value(mainw->blend_layer,"frame",blend_file->frameno);
+  weed_set_int_value(mainw->blend_layer,WEED_LEAF_CLIP,mainw->blend_file);
+  weed_set_int_value(mainw->blend_layer,WEED_LEAF_FRAME,blend_file->frameno);
 
   pull_frame_threaded(mainw->blend_layer,get_image_ext_for_type(blend_file->img_type),tc);
 
@@ -1011,9 +1006,6 @@ weed_plant_t *get_blend_layer(weed_timecode_t tc) {
 // keypresses
 
 
-
-
-
 boolean rte_on_off_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t keyval, LiVESXModifierType mod, livespointer user_data) {
   // this is the callback which happens when a rte is keyed
   int key=LIVES_POINTER_TO_INT(user_data);
@@ -1115,6 +1107,10 @@ boolean rte_on_off_callback_hook(LiVESToggleButton *button, livespointer user_da
 
 boolean grabkeys_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t keyval, LiVESXModifierType mod, livespointer user_data) {
   // assign the keys to the last key-grabbable effect
+  int fx=LIVES_POINTER_TO_INT(user_data);
+  if (fx!=-1) {
+    mainw->last_grabbable_effect=fx;
+  }
   mainw->rte_keys=mainw->last_grabbable_effect;
   mainw->osc_block=TRUE;
   if (rte_window!=NULL) {
@@ -1137,7 +1133,6 @@ boolean textparm_callback(LiVESAccelGroup *group, LiVESObject *obj, uint32_t key
 
 boolean grabkeys_callback_hook(LiVESToggleButton *button, livespointer user_data) {
   if (!lives_toggle_button_get_active(button)) return TRUE;
-  mainw->last_grabbable_effect=LIVES_POINTER_TO_INT(user_data);
   grabkeys_callback(NULL, NULL, 0, (LiVESXModifierType)0, user_data);
   return TRUE;
 }
