@@ -1,6 +1,6 @@
 // liblives.hpp
 // LiVES (lives-exe)
-// (c) G. Finch <salsaman@gmail.com> 2015
+// (c) G. Finch <salsaman@gmail.com> 2015 - 2016
 // Released under the GPL 3 or later
 // see file ../COPYING for licensing details
 
@@ -19,7 +19,7 @@
 /**
    Version number minor
 */
-#define LIVES_VERSION_MINOR 6
+#define LIVES_VERSION_MINOR 8
 
 /**
    Version number micro
@@ -148,7 +148,7 @@ typedef enum {
    Audio sources
 */
 typedef enum {
-  LIVES_AUDIO_SOURCE_UNKNOWN, ///< Unknown / invalid
+  LIVES_AUDIO_SOURCE_UNKNOWN=-1, ///< Unknown / invalid
   LIVES_AUDIO_SOURCE_INTERNAL, ///< Audio source is internal to LiVES
   LIVES_AUDIO_SOURCE_EXTERNAL ///< Audio source is external to LiVES
 } lives_audio_source_t;
@@ -158,7 +158,7 @@ typedef enum {
    Audio players
 */
 typedef enum {
-  LIVES_AUDIO_PLAYER_UNKNOWN, ///< Unknown / invalid
+  LIVES_AUDIO_PLAYER_UNKNOWN=-1, ///< Unknown / invalid
   LIVES_AUDIO_PLAYER_PULSE, ///< Audio playback is through PulseAudio
   LIVES_AUDIO_PLAYER_JACK, ///< Audio playback is thorugh Jack
   LIVES_AUDIO_PLAYER_SOX, ///< Audio playback is through Sox
@@ -657,9 +657,9 @@ namespace lives {
     void invalidate();
     void setClosures(closureList cl);
     
-    bool setPref(int prefidx, bool val) const;
-    bool setPref(int prefidx, int val) const;
-    bool setPref(int prefidx, int bitfield, bool val) const;
+    bool setPref(const char *prefidx, bool val) const;
+    bool setPref(const char *prefidx, int val) const;
+    bool setPref(const char *prefidx, int bitfield, bool val) const;
 
 #endif
 
