@@ -104,6 +104,7 @@ typedef PROCESS_INFORMATION *lives_pgid_t;
 #endif
 #endif
 
+
 #else // IS_MINGW
 
 #ifdef GUI_GTK
@@ -118,6 +119,7 @@ typedef PROCESS_INFORMATION *lives_pgid_t;
 typedef pid_t lives_pid_t;
 typedef int lives_pgid_t;
 
+
 #endif // IS_MINGW
 
 
@@ -174,6 +176,17 @@ typedef int lives_pgid_t;
 
 
 #ifndef IS_MINGW
+#define LIVES_DIR_SEP "/"
+#define LIVES_STATUS_FILE_NAME ".status"
+#define LIVES_INFO_FILE_NAME ".info"
+#define LIVES_BFILE_NAME ".smogrify"
+#define LIVES_SMOGPLUGIN_FILE_NAME ".smogplugin"
+#define LIVES_SMOGVAL_FILE_NAME ".smogval"
+#define LIVES_ENC_DEBUG_FILE_NAME ".debug_out"
+#define LIVES_DEVNULL "/dev/null"
+
+#define DLL_NAME "so"
+
 #define DOC_DIR "/share/doc/lives-"
 
 #define THEME_DIR "/share/lives/themes/"
@@ -181,12 +194,23 @@ typedef int lives_pgid_t;
 #define PLUGIN_COMPOUND_DIR "/share/lives/plugins/"
 #define PLUGIN_EXEC_DIR "/lives/plugins/"
 #define ICON_DIR "/share/lives/icons/"
-#define DESKTOP_ICON_DIR "/share/app-install/icons/"
+#define DESKTOP_ICON_DIR "/share/icons/hicolor/48x48/apps"
 #define DATA_DIR "/share/lives/"
 #define LIVES_CONFIG_DIR ".lives-dir/"
 #define LIVES_TMP_NAME "livestmp"
 
 #else // IS_MINGW
+#define LIVES_DIR_SEP "\\"
+#define LIVES_STATUS_FILE_NAME "status"
+#define LIVES_INFO_FILE_NAME "info"
+#define LIVES_BFILE_NAME "smogrify"
+#define LIVES_SMOGPLUGIN_FILE_NAME "smogplugin"
+#define LIVES_SMOGVAL_FILE_NAME "smogval"
+#define LIVES_ENC_DEBUG_FILE_NAME "debug_out"
+#define LIVES_DEVNULL "NUL"
+
+#define DLL_NAME "dll"
+
 #define DOC_DIR "\\Documents/"
 
 #define THEME_DIR "\\Themes/"
@@ -201,12 +225,14 @@ typedef int lives_pgid_t;
 
 #define LIVES_DEVICE_DIR "/dev/"
 
+#define LIVES_COPYRIGHT_YEARS "2002 - 2016"
 
-#define LIVES_MANUAL_URL "http://lives.sourceforge.net/manual/"
+#define LIVES_WEBSITE PACKAGE_URL
+#define LIVES_MANUAL_URL LIVES_WEBSITE "/manual/"
 #define LIVES_MANUAL_FILENAME "LiVES_manual.html"
-#define LIVES_AUTHOR_EMAIL "mailto:salsaman@gmail.com"
+#define LIVES_AUTHOR_EMAIL "salsaman+lives@gmail.com"
 #define LIVES_DONATE_URL "https://sourceforge.net/p/lives/donate/"
-#define LIVES_BUG_URL "https://sourceforge.net/p/lives/bugs/"
+#define LIVES_BUG_URL PACKAGE_BUGREPORT
 #define LIVES_FEATURE_URL "https://sourceforge.net/p/lives/feature-requests/"
 #define LIVES_TRANSLATE_URL "https://translations.launchpad.net/lives/trunk"
 
@@ -219,7 +245,7 @@ typedef int lives_pgid_t;
 #endif
 #endif
 
-#define DEF_FILE_PERMS S_IRUSR|S_IWUSR // must be at least S_IRUSR|S_IWUSR
+#define DEF_FILE_PERMS (S_IRUSR|S_IWUSR) // must be at least S_IRUSR|S_IWUSR
 #define DEF_FILE_UMASK (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)^( DEF_FILE_PERMS )
 
 /// LiVES will show a warning if this (MBytes) is exceeded on load
@@ -234,8 +260,6 @@ typedef int lives_pgid_t;
 
 #define ENABLE_DVD_GRAB
 
-#define FP_BITS 16 /// max fp bits [apparently 16 is faster]
-
 #ifdef HAVE_MJPEGTOOLS
 #define HAVE_YUV4MPEG
 #endif
@@ -267,8 +291,16 @@ typedef int lives_pgid_t;
 
 #ifdef __GNUC__
 #  define WARN_UNUSED  __attribute__((warn_unused_result))
+#  define GNU_PURE  __attribute__((pure))
+#  define GNU_CONST  __attribute__((const))
+#  define GNU_MALLOC  __attribute__((malloc))
+#  define GNU_ALIGN(x) __attribute__((alloc_align(x)))
 #else
 #  define WARN_UNUSED
+#  define GNU_PURE
+#  define GNU_CONST
+#  define GNU_MALLOC
+#  define GNU_ALIGN(x)
 #endif
 
 #ifdef PRODUCE_LOG
@@ -281,20 +313,22 @@ typedef struct {
   uint16_t red;
   uint16_t green;
   uint16_t blue;
-} lives_colRGB24_t;
+} lives_colRGB48_t;
 
 typedef struct {
   uint16_t red;
   uint16_t green;
   uint16_t blue;
   uint16_t alpha;
-} lives_colRGBA32_t;
+} lives_colRGBA64_t;
 
 
 #include "widget-helper.h"
 
 typedef  void *(*fn_ptr)(void *ptr);
 
+#define ALLOW_MPV
+#define ALLOW_PNG24
 
 /// this struct is used only when physically resampling frames on the disk
 /// we create an array of these and write them to the disk
@@ -461,6 +495,7 @@ typedef enum {
   IMG_TYPE_PNG
 } lives_image_type_t;
 
+#define IMG_TYPE_BEST IMG_TYPE_PNG
 
 
 #define AFORM_SIGNED 0
@@ -480,6 +515,11 @@ typedef enum {
 
 #include "pangotext.h"
 
+#define WEED_LEAF_HOST_DEINTERLACE "host_deinterlace" // frame needs deinterlacing
+#define WEED_LEAF_HOST_TC "host_tc" // timecode for deinterlace
+#define WEED_LEAF_HOST_DECODER "host_decoder" // pointer to decoder for a layer
+#define WEED_LEAF_HOST_PTHREAD "host_pthread" // thread for a layer
+
 /// corresponds to one clip in the GUI
 typedef struct {
   // basic info (saved during backup)
@@ -525,7 +565,7 @@ typedef struct {
   char info_file[PATH_MAX];
   char name[256];  ///< the display name
   char save_file_name[PATH_MAX];
-  char type[40];
+  char type[64];
   int start;
   int end;
   int insert_start;
@@ -663,6 +703,8 @@ typedef struct {
   lives_painter_surface_t *raudio_drawable;
 
   int cb_src; ///< source clip for clipboard
+
+  boolean needs_update; ///< loaded values were incorrect, update header
 } lives_clip_t;
 
 
@@ -703,7 +745,6 @@ typedef struct {
   /// system tempdir (e.g /tmp for linux, C:\TEMP for win32)
   char system_tmpdir[PATH_MAX];  ///< kept in locale encoding
 
-#ifndef IS_MINGW
   char touch_cmd[PATH_MAX];
   char rm_cmd[PATH_MAX];
   char mv_cmd[PATH_MAX];
@@ -712,8 +753,8 @@ typedef struct {
   char chmod_cmd[PATH_MAX];
   char cat_cmd[PATH_MAX];
   char echo_cmd[PATH_MAX];
+  char eject_cmd[PATH_MAX];
   char rmdir_cmd[PATH_MAX];
-#endif
 
   char *rcfile;
 
@@ -855,6 +896,7 @@ int do_info_dialog_with_transient(const char *text, boolean is_blocking, LiVESWi
 LiVESWidget *create_message_dialog(lives_dialog_t diat, const char *text, LiVESWindow *transient,
                                    int warn_mask_number, boolean is_blocking);
 
+LiVESWidget *create_question_dialog(const char *title, const char *text, LiVESWindow *parent);
 
 void do_system_failed_error(const char *com, int retval, const char *addinfo);
 int do_write_failed_error_s_with_retry(const char *fname, const char *errtext, LiVESWindow *transient) WARN_UNUSED;
@@ -888,7 +930,7 @@ void do_mt_backup_space_error(lives_mt *, int memreq_mb);
 boolean do_clipboard_fps_warning(void);
 void perf_mem_warning(void);
 void do_dvgrab_error(void);
-boolean do_comments_dialog(lives_clip_t *sfile, char *filename);
+boolean do_comments_dialog(int fileno, char *filename);
 boolean do_auto_dialog(const char *text, int type);
 void do_encoder_acodec_error(void);
 void do_encoder_sox_error(void);
@@ -896,12 +938,12 @@ boolean rdet_suggest_values(int width, int height, double fps, int fps_num, int
                             int asigned, boolean swap_endian, boolean anr, boolean ignore_fps);
 boolean do_encoder_restrict_dialog(int width, int height, double fps, int fps_num, int fps_denom,
                                    int arate, int asigned, boolean swap_endian, boolean anr, boolean save_all);
-void do_keys_window(void);
-void do_mt_keys_window(void);
 void do_messages_window(void);
 void do_firstever_dialog(void);
 void do_upgrade_error_dialog(void);
 void do_no_mplayer_sox_error(void);
+void do_need_mplayer_dialog(void);
+void do_need_mplayer_mpv_dialog(void);
 void do_aud_during_play_error(void);
 void do_rendered_fx_dialog(void);
 void do_layout_scrap_file_error(void);
@@ -958,6 +1000,9 @@ void do_autolives_needs_clips_error(void);
 void do_pulse_lost_conn_error(void);
 void do_jack_lost_conn_error(void);
 void do_cd_error_dialog(void);
+void do_bad_theme_error(const char *themefile);
+void do_bad_theme_import_error(const char *theme_file);
+boolean do_theme_exists_warn(const char *themename);
 
 boolean process_one(boolean visible);
 void do_threaded_dialog(char *translated_text, boolean has_cancel);
@@ -982,7 +1027,7 @@ void do_text_window(const char *title, const char *text);
 boolean read_file_details(const char *file_name, boolean only_check_for_audio);
 boolean add_file_info(const char *check_handle, boolean aud_only);
 boolean save_file_comments(int fileno);
-boolean reload_clip(int fileno);
+boolean reload_clip(int fileno,int maxframe);
 void reget_afilesize(int fileno);
 ulong deduce_file(const char *filename, double start_time, int end);
 ulong open_file(const char *filename);
@@ -1024,6 +1069,9 @@ int save_to_scrap_file(weed_plant_t *layer);
 boolean load_from_scrap_file(weed_plant_t *layer, int frame);
 void close_ascrap_file(void);
 void close_scrap_file(void);
+void add_to_ascrap_mb(uint64_t bytes);
+
+boolean check_for_disk_space(void);
 
 
 // main.c
@@ -1058,6 +1106,10 @@ LiVESError *lives_pixbuf_save(LiVESPixbuf *pixbuf, char *fname, lives_image_type
 void init_track_decoders(void);
 void free_track_decoders(void);
 
+#ifdef USE_LIBPNG
+boolean layer_from_png(FILE *fp, weed_plant_t *layer, boolean prog);
+boolean save_to_png(FILE *fp, weed_plant_t *layer, int comp);
+#endif
 
 void load_frame_image(int frame);
 void sensitize(void);
@@ -1070,12 +1122,17 @@ void do_quick_switch(int new_file);
 void switch_audio_clip(int new_file, boolean activate);
 void resize(double scale);
 void do_start_messages(void);
-void set_palette_colours(void);
+boolean set_palette_colours(boolean force_reload);
 void set_main_title(const char *filename, int or_untitled_number);
 void set_record(void);
 
 //gui.c
 void  create_LiVES(void);
+void show_lives(void);
+void set_colours(LiVESWidgetColor *colf, LiVESWidgetColor *colb, LiVESWidgetColor *colf2,
+                 LiVESWidgetColor *colb2, LiVESWidgetColor *coli, LiVESWidgetColor *colt);
+void set_preview_box_colours(void);
+void load_theme_images(void);
 void set_interactive(boolean interactive);
 char *get_menu_name(lives_clip_t *sfile);
 void enable_record(void);
@@ -1106,13 +1163,15 @@ boolean expose_eim(LiVESWidget *widget, lives_painter_t *cr, livespointer user_d
 // system calls in utils.c
 int lives_system(const char *com, boolean allow_error);
 lives_pid_t lives_fork(const char *com);
+int lives_open3(const char *pathname, int flags, mode_t mode);
+int lives_open2(const char *pathname, int flags);
 int lives_open_buffered_rdonly(const char *pathname);
 int lives_creat_buffered(const char *pathname, int mode);
 int lives_close_buffered(int fd);
 void lives_close_all_file_buffers(void);
 off_t lives_lseek_buffered_rdonly(int fd, off_t offset);
 ssize_t lives_write(int fd, livesconstpointer buf, size_t count, boolean allow_fail);
-ssize_t lives_write_buffered(int fd, livesconstpointer buf, size_t count, boolean allow_fail);
+ssize_t lives_write_buffered(int fd, const char *buf, size_t count, boolean allow_fail);
 ssize_t lives_write_le(int fd, livesconstpointer buf, size_t count, boolean allow_fail);
 ssize_t lives_write_le_buffered(int fd, livesconstpointer buf, size_t count, boolean allow_fail);
 ssize_t lives_read(int fd, void *buf, size_t count, boolean allow_less);
@@ -1125,7 +1184,9 @@ char *lives_fgets(char *s, int size, FILE *stream);
 pid_t lives_getpid(void);
 int lives_getgid(void);
 int lives_getuid(void);
-void lives_freep(void **ptr);
+boolean lives_freep(void **ptr);
+void lives_kill_subprocesses(const char *dirname, boolean kill_parent);
+void lives_suspend_resume_process(const char *dirname, boolean suspend);
 #ifdef IS_MINGW
 boolean lives_win32_suspend_resume_process(DWORD pid, boolean suspend);
 boolean lives_win32_kill_subprocesses(DWORD pid, boolean kill_parent);
@@ -1138,14 +1199,28 @@ ssize_t lives_readlink(const char *path, char *buf, size_t bufsiz);
 boolean lives_setenv(const char *name, const char *value);
 boolean lives_fsync(int fd);
 void lives_sync(void);
+int lives_rmdir(const char *dir, boolean force);
+int lives_rmdir_with_parents(const char *dir);
+int lives_rm(const char *file);
+int lives_rmglob(const char *files);
+int lives_cp(const char *from, const char *to);
+int lives_cp_keep_perms(const char *from, const char *to);
+int lives_mv(const char *from, const char *to);
+int lives_touch(const char *tfile);
+int lives_chmod(const char *target, const char *mode);
+int lives_cat(const char *from, const char *to, boolean append);
+int lives_echo(const char *text, const char *to, boolean append);
+int lives_ln(const char *from, const char *to);
+
 
 int lives_utf8_strcasecmp(const char *s1, const char *s2);
 
 char *filename_from_fd(char *val, int fd);
 
-float LEFloat_to_BEFloat(float f);
-uint64_t lives_10pow(int pow);
-int get_approx_ln(uint32_t val);
+float LEFloat_to_BEFloat(float f) GNU_CONST;
+uint64_t lives_10pow(int pow) GNU_CONST;
+double lives_fix(double val, int decimals) GNU_CONST;
+int get_approx_ln(uint32_t val) GNU_CONST;
 
 int64_t lives_get_current_ticks(void);
 boolean lives_alarm_get(int alarm_handle);
@@ -1171,6 +1246,9 @@ char *get_val_from_cached_list(const char *key, size_t maxlen);
 void get_location(const char *exe, char *val, int maxlen);
 
 char *make_image_file_name(lives_clip_t *clip, int frame, const char *img_ext);
+const char *get_image_ext_for_type(lives_image_type_t imgtype);
+lives_image_type_t lives_image_ext_to_type(const char *img_ext);
+lives_image_type_t lives_image_type_to_image_type(const char *lives_img_type);
 
 void set_menu_text(LiVESWidget *menu, const char *text, boolean use_mnemonic);
 void get_menu_text(LiVESWidget *menu, char *text);
@@ -1218,18 +1296,22 @@ boolean add_lmap_error(lives_lmap_error_t lerror, const char *name, livespointer
                        int clipno, int frameno, double atime, boolean affects_current);
 void clear_lmap_errors(void);
 boolean prompt_remove_layout_files(void);
+boolean do_std_checks(const char *type_name, const char *type, size_t maxlen, const char *nreject);
 boolean is_legal_set_name(const char *set_name, boolean allow_dupes);
 char *repl_tmpdir(const char *entry, boolean fwd);
 char *clip_detail_to_string(lives_clip_details_t what, size_t *maxlenp);
 boolean get_clip_value(int which, lives_clip_details_t, void *retval, size_t maxlen);
 void save_clip_value(int which, lives_clip_details_t, void *val);
 boolean check_frame_count(int idx);
+void count_opening_frames(void);
 void get_frame_count(int idx);
 void get_frames_sizes(int fileno, int frame_to_test);
 int count_resampled_frames(int in_frames, double orig_fps, double resampled_fps);
 boolean int_array_contains_value(int *array, int num_elems, int value);
 boolean check_for_lock_file(const char *set_name, int type);
-void lives_list_free_strings(LiVESList *list);
+void lives_list_free_strings(LiVESList *);
+void lives_list_free_all(LiVESList **);
+void lives_slist_free_all(LiVESSList **);
 
 boolean create_event_space(int length_in_eventsb);
 void add_to_recent(const char *filename, double start, int frames, const char *file_open_params);
@@ -1265,8 +1347,6 @@ char *insert_newlines(const char *text, int maxwidth);
 int hextodec(char *string);
 int get_hex_digit(const char *c);
 
-const char *get_image_ext_for_type(lives_image_type_t imgtype);
-
 uint32_t fastrand(void);
 void fastsrand(uint32_t seed);
 
@@ -1275,21 +1355,6 @@ int lives_list_strcmp_index(LiVESList *list, livesconstpointer data);
 lives_cancel_t check_for_bad_ffmpeg(void);
 
 //callbacks.c
-void lives_exit(int signum);
-void lives_notify(int msgnumber,const char *msgstring);
-const char *get_set_name(void);
-void count_opening_frames(void);
-void on_fileread_clicked(LiVESFileChooser *, livespointer widget);
-boolean dirchange_callback(LiVESAccelGroup *, LiVESObject *, uint32_t, LiVESXModifierType, livespointer user_data);
-void on_effects_paused(LiVESButton *, livespointer user_data);
-void on_cancel_keep_button_clicked(LiVESButton *, livespointer user_data);
-void on_cleardisk_activate(LiVESWidget *, livespointer user_data);
-void on_cleardisk_advanced_clicked(LiVESWidget *, livespointer user_data);
-void popup_lmap_errors(LiVESMenuItem *, livespointer);
-void on_filesel_button_clicked(LiVESButton *, livespointer user_data);
-void switch_clip(int type, int newclip, boolean force);
-void on_details_button_clicked(void);
-
 
 
 // paramspecial.c
@@ -1297,10 +1362,10 @@ LiVESPixbuf *mt_framedraw(lives_mt *, LiVESPixbuf *);
 
 
 // effects-weed.c
-livespointer _lives_malloc(size_t size);
+livespointer _lives_malloc(size_t size) GNU_MALLOC;
 livespointer lives_memcpy(livespointer dest, livesconstpointer src, size_t n);
 livespointer lives_memset(livespointer s, int c, size_t n);
-void _lives_free(livespointer ptr); ///< calls mainw->free_fn
+void _lives_free(livespointer ptr);
 livespointer lives_calloc(size_t n_blocks, size_t n_block_bytes);
 livespointer _lives_realloc(livespointer ptr, size_t new_size);
 
