@@ -33,6 +33,21 @@
 #define U_SEC_RATIO (U_SECL/1000000) ///< how many U_SECs (ticks) in a microsecond [default 100]
 
 
+#define LIVES_SHORTEST_TIMEOUT  (2. * U_SEC) // 2 sec timeout
+#define LIVES_SHORT_TIMEOUT  (5. * U_SEC) // 5 sec timeout
+#define LIVES_DEFAULT_TIMEOUT  (10. * U_SEC) // 10 sec timeout
+#define LIVES_LONGER_TIMEOUT  (20. * U_SEC) // 20 sec timeout
+#define LIVES_LONGEST_TIMEOUT  (30. * U_SEC) // 30 sec timeout
+
+
+/// rate to change pb fps when faster/slower pressed (TODO: make pref)
+#define PB_CHANGE_RATE .0005
+
+/// forward/back scratch value (TODO: make pref)
+#define PB_SCRATCH_VALUE 0.001
+
+/// adjustment amount for effect parameter via keyboard (TODO: make pref)
+#define BLEND_AMOUNT 0.2
 
 /////// GUI related constants /////////////////////////////////////////////////////////
 
@@ -49,11 +64,18 @@
 /// char width of start / end spinbuttons
 #define SPBWIDTHCHARS 12
 
+
+/// char width of framecounter
+#define FCWIDTHCHARS 18
+
+/// char width of preview spinbutton
+#define PREVSBWIDTHCHARS 8
+
 #define MIN_SEPWIN_WIDTH 600
 #define MIN_SEPWIN_HEIGHT 36
 
 /// sepwin/screen size safety margins in pixels
-#define SCR_HEIGHT_SAFETY ((int)(100.*widget_opts.scale))
+#define SCR_HEIGHT_SAFETY ((int)(200.*widget_opts.scale))
 #define SCR_WIDTH_SAFETY ((int)(100.*widget_opts.scale))
 
 /// default size for generators
@@ -79,6 +101,17 @@
 #define DEFAULT_FRAME_HSIZE ((int)(320.*widget_opts.scale))
 #define DEFAULT_FRAME_VSIZE ((int)(200.*widget_opts.scale))
 
+
+#define FRAMEBLANK_MIN_WIDTH ((int)(240.*widget_opts.scale))
+#define FRAMEBLANK_MAX_WIDTH ((int)(400.*widget_opts.scale))
+
+#define FRAMEBLANK_MIN_HEIGHT ((int)(180.*widget_opts.scale))
+#define FRAMEBLANK_MAX_HEIGHT ((int)(300.*widget_opts.scale))
+
+#define IMSEP_MAX_HEIGHT ((int)(86.*widget_opts.scale))
+#define IMSEP_MAX_WIDTH (mainw->scr_width-20)
+
+
 #define MAIN_SPIN_SPACER ((int)52.*widget_opts.scale) ///< pixel spacing for start/end spins for clip and multitrack editors
 
 /// blank label to show so our message dialogs are not too small
@@ -94,8 +127,6 @@
 #define RFX_WINSIZE_H ((int)(mainw->scr_width>=1024?(820.*widget_opts.scale):640))
 #define RFX_WINSIZE_V ((int)(480.*widget_opts.scale))
 
-#define RFX_TEXT_SCROLL_HEIGHT ((int)(80.*widget_opts.scale)) ///< height of textview scrolled window
-
 #define DEF_BUTTON_WIDTH ((int)(80.*widget_opts.scale))
 
 #define DEF_DIALOG_WIDTH RFX_WINSIZE_H
@@ -159,6 +190,7 @@ typedef enum {
   LIVES_DIALOG_WARN,
   LIVES_DIALOG_WARN_WITH_CANCEL,
   LIVES_DIALOG_YESNO,
+  LIVES_DIALOG_QUESTION,
   LIVES_DIALOG_ABORT_CANCEL_RETRY
 } lives_dialog_t;
 
@@ -189,41 +221,96 @@ typedef enum {
 } lives_storage_status_t;
 
 
+#define DVD_AUDIO_CHAN_MIN 128
+#define DVD_AUDIO_CHAN_DEFAULT 128
+#define DVD_AUDIO_CHAN_MAX 159
+
+enum {
+  LIVES_DEVICE_CD=0,
+  LIVES_DEVICE_DVD, // 1
+  LIVES_DEVICE_VCD, // 2
+  LIVES_DEVICE_INTERNAL, // 2
+  LIVES_DEVICE_TV_CARD, // 4
+  LIVES_DEVICE_FW_CARD, // 5
+};
+
+
+#define THEME_DETAIL_NAME "theme_name"
+#define THEME_DETAIL_STYLE "theme_style"
+#define THEME_DETAIL_SEPWIN_IMAGE "sepwin_image"
+#define THEME_DETAIL_FRAMEBLANK_IMAGE "frameblank_image"
+#define THEME_DETAIL_NORMAL_FORE "normal_fore"
+#define THEME_DETAIL_NORMAL_BACK "normal_back"
+#define THEME_DETAIL_ALT_FORE "alt_fore"
+#define THEME_DETAIL_ALT_BACK "alt_back"
+#define THEME_DETAIL_INFO_TEXT "info_text"
+#define THEME_DETAIL_INFO_BASE "info_base"
+
+#define THEME_DETAIL_AUDCOL "audcol"
+#define THEME_DETAIL_VIDCOL "vidcol"
+#define THEME_DETAIL_FXCOL "fxcol"
+#define THEME_DETAIL_MT_TLREG "mt_tlreg"
+#define THEME_DETAIL_MT_MARK "mt_mark"
+#define THEME_DETAIL_MT_EVBOX "mt_evbox"
+#define THEME_DETAIL_MT_TCFG "mt_timecode_fg"
+#define THEME_DETAIL_MT_TCBG "mt_timecode_bg"
+#define THEME_DETAIL_FRAME_SURROUND "frame_surround"
+#define THEME_DETAIL_CE_SEL "ce_sel"
+#define THEME_DETAIL_CE_UNSEL "ce_unsel"
+
 
 /// set in set_palette_colours()
 typedef struct {
   int style;
 #define STYLE_PLAIN 0 ///< no theme (theme 'none')
-#define STYLE_1 1<<0 ///< turn on theming if set
-#define STYLE_2 1<<1 ///< colour the spinbuttons on the front page if set
-#define STYLE_3 1<<2 ///< style is lightish - allow themeing of widgets with dark text, otherwise use menu bg
-#define STYLE_4 1<<3 ///< coloured bg for poly window in mt
-#define STYLE_5 1<<4 ///< drop down menu text col. in mt
-#define STYLE_6 1<<4 ///< separator col. in mt
+#define STYLE_1 (1<<0) ///< turn on theming if set
+#define STYLE_2 (1<<1) ///< colour the spinbuttons on the front page if set
+#define STYLE_3 (1<<2) ///< style is lightish - allow themeing of widgets with dark text, otherwise use menu bg
+#define STYLE_4 (1<<3) ///< separator col. in mt
 
   LiVESWidgetColor white;
   LiVESWidgetColor black;
-  LiVESWidgetColor light_blue;
-  LiVESWidgetColor light_yellow;
   LiVESWidgetColor pink;
   LiVESWidgetColor light_red;
-  LiVESWidgetColor dark_red;
   LiVESWidgetColor light_green;
+  LiVESWidgetColor dark_red;
+  LiVESWidgetColor dark_orange;
+
   LiVESWidgetColor grey20;
   LiVESWidgetColor grey25;
   LiVESWidgetColor grey45;
   LiVESWidgetColor grey60;
-  LiVESWidgetColor dark_orange;
   LiVESWidgetColor fade_colour;
+
+  LiVESWidgetColor banner_fade_text;
+
+  // set via theme API
+
+
   LiVESWidgetColor normal_back;
   LiVESWidgetColor normal_fore;
 
   LiVESWidgetColor menu_and_bars;
   LiVESWidgetColor menu_and_bars_fore;
-  LiVESWidgetColor banner_fade_text;
   LiVESWidgetColor info_text;
   LiVESWidgetColor info_base;
 
+  LiVESWidgetColor mt_timecode_bg;
+  LiVESWidgetColor mt_timecode_fg;
+
+  lives_colRGBA64_t audcol;
+  lives_colRGBA64_t vidcol;
+  lives_colRGBA64_t fxcol;
+  lives_colRGBA64_t mt_timeline_reg;
+
+  lives_colRGBA64_t frame_surround;
+  lives_colRGBA64_t mt_mark;
+  lives_colRGBA64_t mt_evbox;
+
+  lives_colRGBA64_t ce_sel;
+  lives_colRGBA64_t ce_unsel;
+
+
 } _palette;
 
 /// screen details
@@ -251,12 +338,37 @@ enum {
 };
 
 
+// image types (string)
+#define LIVES_IMAGE_TYPE_JPEG "jpeg"
+#define LIVES_IMAGE_TYPE_PNG "png"
+
+// audio types (string)
+#define LIVES_AUDIO_TYPE_PCM "pcm"
+
+
 // file extensions
 #define LIVES_FILE_EXT_PNG "png"
 #define LIVES_FILE_EXT_JPG "jpg"
 #define LIVES_FILE_EXT_MGK "mgk"
 #define LIVES_FILE_EXT_PRE "pre"
 #define LIVES_FILE_EXT_SCRAP "scrap"
+#define LIVES_FILE_EXT_TEXT "txt"
+#define LIVES_FILE_EXT_BAK "bak"
+
+#define LIVES_FILE_EXT_BACKUP "lv1"
+#define LIVES_FILE_EXT_PROJECT "lv2"
+
+#define LIVES_FILE_EXT_TAR_GZ "tar.gz"
+
+#define LIVES_FILE_EXT_SRT "srt"
+#define LIVES_FILE_EXT_SUB "sub"
+
+#define LIVES_FILE_EXT_WAV "wav"
+
+// filters
+
+#define LIVES_SUBS_FILTER  {"*.srt","*.sub",NULL}
+#define LIVES_AUDIO_LOAD_FILTER  {"*.it","*.mp3","*.wav","*.ogg","*.mod","*.xm","*.wma","*.flac",NULL}
 
 
 typedef struct {
@@ -303,7 +415,8 @@ typedef struct {
   LiVESSList *clips_group;
 
   /// sets
-  char set_name[256];   // actually 128 is the limit now, filesystem encoding
+#define MAX_SET_NAME_LEN 128
+  char set_name[256];   // actually 128 is the (soft) limit now, filesystem encoding
 
   // playback
   boolean faded;
@@ -462,9 +575,7 @@ typedef struct {
 
   // prefs (Save on exit)
   int prefs_changed;
-#define PREFS_THEME_CHANGED (1<<0)
-#define PREFS_JACK_CHANGED (1<<1)
-#define PREFS_TEMPDIR_CHANGED (1<<2)
+
   boolean prefs_need_restart;
 
   /// default sizes for when no file is loaded
@@ -671,6 +782,8 @@ typedef struct {
   LiVESWidget *close;
   LiVESWidget *import_proj;
   LiVESWidget *export_proj;
+  LiVESWidget *import_theme;
+  LiVESWidget *export_theme;
   LiVESWidget *sw_sound;
   LiVESWidget *clear_ds;
   LiVESWidget *ccpd_sound;
@@ -820,6 +933,7 @@ typedef struct {
   LiVESWidget *preview_image;
   LiVESWidget *preview_spinbutton;
   LiVESWidget *preview_scale;
+  LiVESWidget *preview_hbox;
   int preview_frame;
   ulong preview_spin_func;
   int prv_link;
@@ -932,7 +1046,6 @@ typedef struct {
   LiVESWidget *custom_utilities_submenu;
   LiVESWidget *custom_utilities_separator;
   LiVESWidget *rte_separator;
-  LiVESWidget *invis;
 
   int num_tracks;
   int *clip_index;
@@ -1081,6 +1194,14 @@ typedef struct {
   LiVESWidget *splash_label;
   LiVESWidget *splash_progress;
 
+#define SPLASH_LEVEL_BEGIN .0
+#define SPLASH_LEVEL_START_GUI .2
+#define SPLASH_LEVEL_LOAD_RTE .4
+#define SPLASH_LEVEL_LOAD_APLAYER .6
+#define SPLASH_LEVEL_LOAD_RFX .8
+#define SPLASH_LEVEL_COMPLETE 1.
+
+
   boolean recoverable_layout;
 
   boolean soft_debug; ///< for testing
@@ -1201,6 +1322,12 @@ typedef struct {
   boolean interactive;
 
   int fc_buttonresponse;
+
+  char frameblank_path[PATH_MAX];
+  char sepimg_path[PATH_MAX];
+
+  uint64_t aud_data_written;
+
   ////////////////////
 
 } mainwindow;
