@@ -27,6 +27,7 @@ LiVESList *get_plugin_list(const char *plugin_type, boolean allow_nonex, const c
 
 /// smogrify handles the directory differently for themes
 #define PLUGIN_THEMES "themes"
+#define PLUGIN_THEMES_CUSTOM "custom/themes"
 
 /// uses WEED_PLUGIN_PATH
 #define PLUGIN_EFFECTS_WEED "weed"
@@ -77,9 +78,6 @@ typedef struct {
   ///< optional (but should return a weed plantptr array of paramtmpl and chantmpl, NULL terminated)
   const weed_plant_t **(*get_play_params)(weed_bootstrap_f f);
 
-  // only for display plugins
-  boolean(*send_keycodes)(plugin_keyfunc);
-
   // optional for YUV palettes
   int *(*get_yuv_palette_sampling)(int palette);
   int *(*get_yuv_palette_clamping)(int palette);
@@ -97,9 +95,9 @@ typedef struct {
 
   uint64_t capabilities;
 
-#define VPP_CAN_RESIZE    1<<0
-#define VPP_CAN_RETURN    1<<1
-#define VPP_LOCAL_DISPLAY 1<<2
+#define VPP_CAN_RESIZE    (1<<0)
+#define VPP_CAN_RETURN    (1<<1)
+#define VPP_LOCAL_DISPLAY (1<<2)
 
   int fwidth,fheight;
 
@@ -140,12 +138,21 @@ const weed_plant_t *pp_get_chan(weed_plant_t **pparams, int idx);
 
 // encoder plugins
 
+#define MULTI_ENCODER_NAME "multi_encoder"
+#define MULTI_ENCODER3_NAME "multi_encoder3"
+
+#define HI_THEORA_FORMAT "hi-theora"
+#define HI_MPEG_FORMAT "hi-mpg"
+#define HI_H_MKV_FORMAT "hi_h-mkv"
+#define HI_H_AVI_FORMAT "hi_h-avi"
+
+
 void do_plugin_encoder_error(const char *plugin_name_or_null);
 
 LiVESList *filter_encoders_by_img_ext(LiVESList *encoders, const char *img_ext);
 
 typedef struct {
-  char name[51];
+  char name[64];
   uint32_t audio_codec;
   // match with bitmaps in the encoder plugins
   // and also anames array in plugins.c (see below)
@@ -169,13 +176,13 @@ typedef struct {
   uint32_t capabilities;
 
 
-#define HAS_RFX 1<<0
+#define HAS_RFX (1<<0)
 
-#define CAN_ENCODE_PNG 1<<2
-#define ENCODER_NON_NATIVE 1<<3
+#define CAN_ENCODE_PNG (1<<2)
+#define ENCODER_NON_NATIVE (1<<3)
 
   // current output format
-  char of_name[51];
+  char of_name[64];
   char of_desc[128];
   int of_allowed_acodecs;
   char of_restrict[1024];
@@ -501,6 +508,10 @@ typedef struct {
 
   LiVESWidget *menuitem;  ///< the menu item which activates this effect
   int num_params;
+  uint32_t flags; /// internal use
+#define RFX_FLAGS_NO_SLIDERS 0x0001
+
+
   lives_param_t *params;
   lives_rfx_source_t source_type;
   void *source;  ///< points to the source (e.g. a weed_plant_t)
@@ -553,8 +564,8 @@ typedef struct {
 boolean get_bool_param(void *value);
 int get_int_param(void *value);
 double get_double_param(void *value);
-void get_colRGB24_param(void *value, lives_colRGB24_t *rgb);
-void get_colRGBA32_param(void *value, lives_colRGBA32_t *rgba);
+void get_colRGB24_param(void *value, lives_colRGB48_t *rgb);
+void get_colRGBA32_param(void *value, lives_colRGBA64_t *rgba);
 
 void set_bool_param(void *value, boolean);
 void set_int_param(void *value, int);
