@@ -1,16 +1,25 @@
 // preferences.h
 // LiVES (lives-exe)
-// (c) G. Finch (salsaman@gmail.com) 2004 - 2013
+// (c) G. Finch (salsaman@gmail.com) 2004 - 2016
 // released under the GNU GPL 3 or later
 // see file ../COPYING or www.gnu.org for licensing details
 
 #ifndef HAS_LIVES_PREFS_H
 #define HAS_LIVES_PREFS_H
 
-#define LIVES_PREFS_TIMEOUT  (10 * U_SEC) // 10 sec timeout
-
 #define PREFS_PANED_POS ((int)(200.*widget_opts.scale))
 
+// for mainw->prefs_changed
+#define PREFS_THEME_CHANGED (1<<0)
+#define PREFS_JACK_CHANGED (1<<1)
+#define PREFS_TEMPDIR_CHANGED (1<<2)
+#define PREFS_COLOURS_CHANGED (1<<3)
+#define PREFS_XCOLOURS_CHANGED (1<<4)
+#define PREFS_IMAGES_CHANGED (1<<5)
+
+#define PULSE_AUDIO_URL "http://www.pulseaudio.org"
+#define JACK_URL "http://jackaudio.org"
+
 typedef struct {
   char bgcolour[256];
   boolean stop_screensaver;
@@ -31,6 +40,17 @@ typedef struct {
 #define AUD_PLAYER_PULSE 4
 #define AUD_PLAYER_MPLAYER2 5
 
+  // string forms
+#define AUDIO_PLAYER_SOX "sox"
+#define AUDIO_PLAYER_JACK "jack"
+#define AUDIO_PLAYER_PULSE "pulse"
+#define AUDIO_PLAYER_PULSE_AUDIO "pulse audio" // display version
+
+#define AUDIO_PLAYER_MPLAYER "mplayer"
+#define AUDIO_PLAYER_MPLAYER2 "mplayer2"
+
+
+
   char aplayer[512]; // name, eg. "jack","pulse","sox","mplayer","mplayer2"
 
   /// frame quantisation type
@@ -101,10 +121,10 @@ typedef struct {
 #define WARN_MASK_NO_PULSE_CONNECT (1<<25)
 
 #define WARN_MASK_LAYOUT_WIPE (1<<26)
-
-  char effect_command[256];
-  char video_open_command[256];
-  char audio_play_command[256];
+  char cmd_log[PATH_MAX];
+  char effect_command[PATH_MAX*2];
+  char video_open_command[PATH_MAX*2];
+  char audio_play_command[PATH_MAX*2];
   char cdplay_device[PATH_MAX];  ///< locale encoding
   double default_fps;
   int bar_height;
@@ -172,8 +192,8 @@ typedef struct {
 #define JACK_OPTS_TIMEBASE_MASTER (1<<7)   ///< timebase master (not implemented yet)
 #define JACK_OPTS_NO_READ_AUTOCON (1<<8)   ///< do not auto connect read clients when playing ext audio
 
-  char jack_tserver[256];
-  char jack_aserver[256];
+  char jack_tserver[PATH_MAX];
+  char jack_aserver[PATH_MAX];
 
   char *fxdefsfile;
   char *fxsizesfile;
@@ -211,7 +231,7 @@ typedef struct {
   boolean mt_pertrack_audio;
   int mt_backaudio;
 
-  int mt_auto_back;
+  int mt_auto_back; ///< time diff to backup (-1 == never, 0 == after every change, >0 == seconds)
 
   boolean ar_clipset;
   boolean ar_layout;
@@ -246,9 +266,9 @@ typedef struct {
   int midi_check_rate;
   int midi_rpt;
 
-#define OMC_DEV_MIDI 1<<0
-#define OMC_DEV_JS 1<<1
-#define OMC_DEV_FORCE_RAW_MIDI 1<<2
+#define OMC_DEV_MIDI (1<<0)
+#define OMC_DEV_JS (1<<1)
+#define OMC_DEV_FORCE_RAW_MIDI (1<<2)
   uint32_t omc_dev_opts;
 
   char omc_js_fname[PATH_MAX];  ///< utf8
@@ -350,6 +370,8 @@ typedef struct {
 
   int max_disp_vtracks;
 
+  boolean mt_load_fuzzy;
+
 } _prefs;
 
 enum {
@@ -377,8 +399,8 @@ enum {
 };
 
 
-#define PREF_WIN_WIDTH (mainw->scr_width*.9)
-#define PREF_WIN_HEIGHT (mainw->scr_height*.9)
+#define PREFWIN_WIDTH (mainw->scr_width*.9)
+#define PREFWIN_HEIGHT (mainw->scr_height*.9)
 
 #define DS_WARN_CRIT_MAX 1000000. ///< MB.
 
@@ -447,6 +469,7 @@ typedef struct {
   LiVESWidget *reffects;
   LiVESWidget *raudio;
   LiVESWidget *rextaudio;
+  LiVESWidget *rintaudio;
   LiVESWidget *rdesk_audio;
   LiVESWidget *encoder_combo;
   LiVESWidget *checkbutton_antialias;
@@ -482,6 +505,28 @@ typedef struct {
   LiVESWidget *spinbutton_warn_fsize;
   LiVESWidget *spinbutton_bwidth;
   LiVESWidget *theme_combo;
+  LiVESWidget *cbutton_fore;
+  LiVESWidget *cbutton_back;
+  LiVESWidget *cbutton_mabf;
+  LiVESWidget *cbutton_mab;
+  LiVESWidget *cbutton_infot;
+  LiVESWidget *cbutton_infob;
+  LiVESWidget *theme_style2;
+  LiVESWidget *theme_style3;
+  LiVESWidget *theme_style4;
+
+  LiVESWidget *cbutton_fsur;
+  LiVESWidget *cbutton_evbox;
+  LiVESWidget *cbutton_mtmark;
+  LiVESWidget *cbutton_tlreg;
+  LiVESWidget *cbutton_tcfg;
+  LiVESWidget *cbutton_tcbg;
+  LiVESWidget *cbutton_vidcol;
+  LiVESWidget *cbutton_audcol;
+  LiVESWidget *cbutton_fxcol;
+  LiVESWidget *cbutton_cesel;
+  LiVESWidget *cbutton_ceunsel;
+
   LiVESWidget *check_midi;
   LiVESWidget *ins_speed;
   LiVESWidget *jpeg;
@@ -554,7 +599,11 @@ typedef struct {
   LiVESWidget *ladspa_entry;
   LiVESWidget *cdda_hbox;
   LiVESWidget *midi_hbox;
+  LiVESWidget *frameblank_entry;
+  LiVESWidget *sepimg_entry;
   LiVESTreeSelection *selection;
+  
+  boolean ignore_apply; ///< dont light the apply button when thing changes (for external calls), normally FALSE
   boolean needs_restart;
 } _prefsw;
 
@@ -613,6 +662,8 @@ void on_prefs_close_clicked(LiVESButton *, livespointer);
 
 void on_prefs_revert_clicked(LiVESButton *, livespointer);
 
+void on_prefs_apply_clicked(LiVESButton *, livespointer user_data);
+
 void set_vpp(boolean set_in_prefs);
 
 void on_prefDomainChanged(LiVESTreeSelection *, livespointer);
@@ -624,39 +675,222 @@ void set_combo_box_active_string(LiVESCombo *, char *active_str);
 void prefsw_set_astream_settings(_vid_playback_plugin *);
 void prefsw_set_rec_after_settings(_vid_playback_plugin *);
 
+void pref_change_images(void);
+void pref_change_xcolours(void);
+void pref_change_colours(void);
+
+
 void apply_button_set_enabled(LiVESWidget *widget, livespointer func_data);
 
 
-// factories
-enum {
-  PREF_REC_EXT_AUDIO,
-  PREF_AUDIO_OPTS,
-  PREF_SEPWIN_STICKY,
-  PREF_MT_EXIT_RENDER
-};
+// factories (pseudo prefs), called from cpp binding
 
-void pref_factory_bool(int prefidx, boolean newval);
-void pref_factory_int(int prefidx, int newval);
-void pref_factory_bitmapped(int prefidx, int bitfield, boolean newval);
+#define PREF_REC_EXT_AUDIO "rec_ext_audio"
+#define PREF_AUDIO_OPTS "audio_opts"
+#define PREF_SEPWIN_STICKY "sepwin_sticky"
+#define PREF_MT_EXIT_RENDER "mt_exit_render"
 
 
-void get_pref(const char *key, char *val, int maxlen);
-void get_pref_utf8(const char *key, char *val, int maxlen);
+
+// normal prefs
+
+/////////////////// string values
+
+#define PREF_WORKING_DIR "tempdir"
+#define PREF_PREFIX_DIR "prefix_dir" // readonly
+#define PREF_LIB_DIR "lib_dir" // readonly
+
+#define PREF_SESSION_TEMPDIR "session_tempdir"
+
+#define PREF_AUDIO_PLAYER "audio_player"
+#define PREF_AUDIO_PLAY_COMMAND "audio_play_command"
+
+#define PREF_AUDIO_SRC "audio_src"
+
+#define PREF_MONITORS "monitors"
+
+#define PREF_LADSPA_PATH "ladspa_path"
+#define PREF_WEED_PLUGIN_PATH "weed_plugin_path"
+#define PREF_FREI0R_PATH "frei0r_path"
+
+#define PREF_VID_PLAYBACK_PLUGIN "vid_playback_plugin"
+
+#define PREF_DEFAULT_IMAGE_FORMAT "default_image_format"
+
+#define PREF_VIDEO_OPEN_COMMAND "video_open_command"
+
+#define PREF_GUI_THEME "gui_theme"
+
+#define PREF_ENCODER "encoder"
+#define PREF_OUTPUT_TYPE "output_type"
+
+#define PREF_CDPLAY_DEVICE "cdplay_device"
+
+#define PREF_AR_LAYOUT "ar_layout"
+#define PREF_AR_CLIPSET "ar_clipset"
+
+#define PREF_CURRENT_AUTOTRANS "current_autotrans"
+#define PREF_DEF_AUTOTRANS "def_autotrans"  // readonly
+
+
+////////////////////// utf8 values
+
+#define PREF_OMC_MIDI_FNAME "omc_midi_fname"
+#define PREF_OMC_JS_FNAME "omc_js_fname"
+
+#define PREF_IMAGE_DIR "image_dir"
+#define PREF_AUDIO_DIR "audio_dir"
+
+#define PREF_PROJ_DIR "proj_dir"
+
+#define PREF_VID_SAVE_DIR "vid_save_dir"
+#define PREF_VID_LOAD_DIR "vid_load_dir"
+
+#define PREF_RECENT1 "recent1"
+#define PREF_RECENT2 "recent2"
+#define PREF_RECENT3 "recent3"
+#define PREF_RECENT4 "recent4"
+
+
+/////////////////// integer64 values
+#define PREF_DS_WARN_LEVEL "ds_warn_level"
+#define PREF_DS_CRIT_LEVEL "ds_crit_level"
+
+
+/////////////////// integer32 values
+#define PREF_STARTUP_PHASE "startup_phase"
+
+#define PREF_STARTUP_INTERFACE "startup_interface"
+
+#define PREF_LIVES_WARNING_MASK "lives_warning_mask"
+#define PREF_OPEN_COMPRESSION_PERCENT "open_compression_percent"
+
+#define PREF_PB_QUALITY "pb_quality"
+
+#define PREF_REC_STOP_GB "rec_stop-gb"
+
+#define PREF_NFX_THREADS "nfx_threads"
+
+#define PREF_CLEAR_DISK_OPTS "clear_disk_opts"
+
+#define PREF_MAX_DISP_VTRACKS "max_disp_vtracks"
+
+#define PREF_RTE_KEYS_VIRTUAL "rte_keys_virtual"
+
+#define PREF_JACK_OPTS "jack_opts"
+
+#define PREF_MIDI_CHECK_RATE "midi_check_rate"
+#define PREF_MIDI_RPT "midi_rpt"
+
+#define PREF_ENCODER_ACODEC "encoder_acodec"
+
+#define PREF_AUDIO_OPTS "audio_opts"
+
+#define PREF_RECORD_OPTS "record_opts"
+
+#define PREF_OMC_DEV_OPTS "omc_dev_opts"
+#define PREF_OSC_PORT "osc_port"
+
+#define PREF_MT_DEF_WIDTH "mt_def_width"
+#define PREF_MT_DEF_HEIGHT "mt_def_height"
+#define PREF_MT_DEF_ARATE "mt_def_arate"
+#define PREF_MT_DEF_ACHANS "mt_def_achans"
+#define PREF_MT_DEF_ASAMPS "mt_def_asamps"
+#define PREF_MT_DEF_SIGNED_ENDIAN "mt_def_signed_endian"
+
+#define PREF_MT_AUTO_BACK "mt_auto_back"
+#define PREF_MT_UNDO_BUF "mt_undo_buf"
+
+#define PREF_MT_BACKAUDIO "mt_backaudio"
+
+#define PREF_WARN_FILE_SIZE "warn_file_size"
+
+#define PREF_DL_BANDWIDTH_K "dl_bandwidth_K"
+
+
+////////// boolean values
+#define PREF_SHOW_RECENT_FILES "show_recent_files"
+#define PREF_FORCE_SINGLE_MONITOR "force_single_monitor"
+#define PREF_STOP_SCREENSAVER "stop_screensaver"
+#define PREF_MT_ENTER_PROMPT "mt_enter_prompt"
+#define PREF_MT_EXIT_RENDER "mt_exit_render"
+#define PREF_RENDER_PROMPT "render_prompt"
+#define PREF_MT_PERTRACK_AUDIO "mt_pertrack_audio"
+#define PREF_OSC_START "osc_start"
+#define PREF_SHOW_TOOLBAR "show_toolbar"
+#define PREF_CE_MAXSPECT "ce_maxspect"
+#define PREF_OPEN_MAXIMISED "open_maximised"
+#define PREF_AUTO_TRIM_PAD_AUDIO "auto_trim_pad_audio"
+#define PREF_MOUSE_SCROLL_CLIPS "mouse_scroll_clips"
+#define PREF_SHOW_BUTTON_ICONS "show_button_icons"
+#define PREF_STREAM_AUDIO_OUT "stream_audio_out"
+#define PREF_CE_THUMB_MODE "ce_thumb_mode"
+#define PREF_ANTIALIAS "antialias"
+#define PREF_FILESEL_MAXIMISED "filesel_maximised"
+#define PREF_SHOW_PLAYER_STATS "show_player_stats"
+#define PREF_INSTANT_OPEN "instant_open"
+#define PREF_MIDISYNCH "midisynch"
+#define PREF_AUTO_DEINTERLACE "auto_deinterlace"
+#define PREF_AUTO_CUT_BORDERS "auto_cut_borders"
+#define PREF_REC_DESKTOP_AUDIO "rec_desktop_audio"
+#define PREF_INSERT_RESAMPLE "insert_resample"
+#define PREF_CONCAT_IMAGES "concat_images"
+#define PREF_SAVE_DIRECTORIES "save_directories"
+#define PREF_CONSERVE_SPACE "conserve_space"
+
+
+
+////////// double values
+#define PREF_MT_DEF_FPS "mt_def_fps"
+
+#define PREF_DEFAULT_FPS "default_fps"
+
+
+
+////////// list values
+#define PREF_DISABLED_DECODERS "disabled_decoders"
+
+
+
+// defaults (readonly)
+/////////////// string values
+#define PREF_SOX_COMMAND "sox_command"
+#define PREF_MPLAYER_AUDIO_COMMAND "mplayer_audio_command"
+#define PREF_MPLAYER2_AUDIO_COMMAND "mplayer2_audio_command"
+
+
+
+void pref_factory_bool(const char *prefidx, boolean newval);
+void pref_factory_int(const char *prefidx, int newval);
+void pref_factory_bitmapped(const char *prefidx, int bitfield, boolean newval);
+
+
+int get_pref(const char *key, char *val, int maxlen);
+int get_pref_from_file(const char *filename, const char *key, char *val, int maxlen);
+int get_pref_utf8(const char *key, char *val, int maxlen);
 void get_pref_default(const char *key, char *val, int maxlen);
 boolean get_boolean_pref(const char *key);
 double get_double_pref(const char *key);
 int get_int_pref(const char *key);
 LiVESList *get_list_pref(const char *key);
+boolean get_colour_pref(const char *key, lives_colRGBA64_t *lcol);
+boolean get_theme_colour_pref(const char *themefile, const char *key, lives_colRGBA64_t *lcol);
 void set_pref(const char *key, const char *value);
+void set_pref_utf8(const char *key, const char *value);
 void delete_pref(const char *key);
 void set_boolean_pref(const char *key, boolean value);
 void set_double_pref(const char *key, double value);
 void set_int_pref(const char *key, int value);
 void set_int64_pref(const char *key, int64_t value);
 void set_list_pref(const char *key, LiVESList *values);
+void set_colour_pref(const char *key, lives_colRGBA64_t *lcol);
+void set_theme_pref(const char *themefile, const char *key, const char *value);
+void set_theme_colour_pref(const char *themefile, const char *key, lives_colRGBA64_t *lcol);
 boolean apply_prefs(boolean skip_warnings);
 void save_future_prefs(void);
 
+void set_palette_prefs(void);
+
 // permissions
 
 #define LIVES_PERM_OSC_PORTS 1
