@@ -36,6 +36,14 @@ static boolean seek_err;
 
 ///////////////////////////////////////////////////////////////////
 
+void pa_mloop_lock(void) {
+  pa_threaded_mainloop_lock(pa_mloop);
+}
+
+void pa_mloop_unlock(void) {
+  pa_threaded_mainloop_unlock(pa_mloop);
+}
+
 
 static void pulse_server_cb(pa_context *c,const pa_server_info *info, void *userdata) {
   if (info==NULL) {
@@ -45,9 +53,6 @@ static void pulse_server_cb(pa_context *c,const pa_server_info *info, void *user
   pulse_server_rate=info->sample_spec.rate;
 }
 
-// wait 5 seconds to startup
-#define PULSE_START_WAIT 500000000
-
 
 boolean lives_pulse_init(short startup_phase) {
   // startup pulse audio server
@@ -68,14 +73,14 @@ boolean lives_pulse_init(short startup_phase) {
 
   stime=lives_get_current_ticks();
 
-  while (pa_state!=PA_CONTEXT_READY&&ntime<PULSE_START_WAIT) {
+  while (pa_state!=PA_CONTEXT_READY&&ntime<LIVES_SHORT_TIMEOUT) {
     lives_usleep(prefs->sleep_time);
     sched_yield();
     pa_state=pa_context_get_state(pcon);
     ntime=lives_get_current_ticks()-stime;
   }
 
-  if (ntime>=PULSE_START_WAIT) {
+  if (ntime>=LIVES_SHORT_TIMEOUT) {
     pa_context_unref(pcon);
     pcon=NULL;
     pulse_shutdown();
@@ -226,16 +231,13 @@ static void pulse_audio_write_process(pa_stream *pstream, size_t nbytes, void *a
         if (pulsed->is_opening) filename=lives_build_filename(prefs->tmpdir,mainw->files[new_file]->handle,
                                            "audiodump.pcm",NULL);
         else filename=lives_build_filename(prefs->tmpdir,mainw->files[new_file]->handle,"audio",NULL);
-        pulsed->fd=open(filename,O_RDONLY);
+        pulsed->fd=lives_open2(filename,O_RDONLY);
         if (pulsed->fd==-1) {
           // dont show gui errors - we are running in realtime thread
           LIVES_ERROR("pulsed: error opening");
           LIVES_ERROR(filename);
           pulsed->playing_file=-1;
         } else {
-#ifdef IS_MINGW
-          setmode(pulsed->fd, O_BINARY);
-#endif
           pulsed->seek_pos=0;
           pulsed->playing_file=new_file;
           pulsed->audio_ticks=mainw->currticks;
@@ -248,8 +250,7 @@ static void pulse_audio_write_process(pa_stream *pstream, size_t nbytes, void *a
     case ASERVER_CMD_FILE_CLOSE:
       if (pulsed->fd>=0) close(pulsed->fd);
       pulsed->fd=-1;
-      if (pulsed->aPlayPtr->data!=NULL) lives_free(pulsed->aPlayPtr->data);
-      pulsed->aPlayPtr->data=NULL;
+      lives_freep((void **)&pulsed->aPlayPtr->data);
       pulsed->aPlayPtr->max_size=0;
       pulsed->aPlayPtr->size=0;
       pulsed->playing_file=-1;
@@ -269,7 +270,7 @@ static void pulse_audio_write_process(pa_stream *pstream, size_t nbytes, void *a
     default:
       msg->data=NULL;
     }
-    if (msg->data!=NULL) lives_free((livespointer)msg->data);
+    lives_freep((void **)&msg->data);
     msg->command=ASERVER_CMD_PROCESSED;
     if (msg->next==NULL) pulsed->msgq=NULL;
     else pulsed->msgq = msg->next;
@@ -635,7 +636,7 @@ static void pulse_audio_write_process(pa_stream *pstream, size_t nbytes, void *a
                                     pulsed->out_achans,16,0,(capable->byte_order==LIVES_LITTLE_ENDIAN),FALSE,1.0);
             }
 
-            if (fbuffer!=NULL) lives_free(fbuffer);
+            lives_freep((void **)&fbuffer);
             free(fp);
 
             if (mainw->record&&mainw->ascrap_file!=-1&&mainw->playing_file>0) {
@@ -716,8 +717,7 @@ static void pulse_audio_write_process(pa_stream *pstream, size_t nbytes, void *a
       }
 
       if (needs_free&&pulsed->sound_buffer!=pulsed->aPlayPtr->data&&pulsed->sound_buffer!=NULL) {
-        lives_free(pulsed->sound_buffer);
-        pulsed->sound_buffer=NULL;
+        lives_freep((void **)&pulsed->sound_buffer);
       }
     }
 
@@ -814,7 +814,16 @@ size_t pulse_flush_read_data(pulse_driver_t *pulsed, int fileno, size_t rbytes,
     size_t target=frames_out*(ofile->asampsize/8)*ofile->achans,bytes;
     // use write not lives_write - because of potential threading issues
     bytes=write(mainw->aud_rec_fd,holding_buff,target);
-    if (bytes<target) mainw->bad_aud_file=filename_from_fd(NULL,mainw->aud_rec_fd);
+    if (bytes>0) {
+      mainw->aud_data_written+=bytes;
+      if (mainw->ascrap_file!=-1&&mainw->files[mainw->ascrap_file]!=NULL&&mainw->aud_rec_fd==mainw->files[mainw->ascrap_file]->cb_src)
+        add_to_ascrap_mb(bytes);
+      if (mainw->aud_data_written>AUD_WRITTEN_CHECK) {
+        mainw->aud_data_written=0;
+        check_for_disk_space();
+      }
+      if (bytes<target) mainw->bad_aud_file=filename_from_fd(NULL,mainw->aud_rec_fd);
+    }
   }
 
   lives_free(holding_buff);
@@ -848,7 +857,7 @@ static void pulse_audio_read_process(pa_stream *pstream, size_t nbytes, void *ar
   if (pulsed->playing_file==-1) {
     out_scale=1.0; // just listening, no recording
   } else {
-    out_scale=(float)pulsed->in_arate/(float)afile->arate; // recording to ascrap_file
+    out_scale=(float)afile->arate/(float)pulsed->in_arate; // recording to ascrap_file
     prb+=rbytes;
   }
 
@@ -1228,7 +1237,7 @@ int64_t lives_pulse_get_time(pulse_driver_t *pulsed, boolean absolute) {
 
   if (msg!=NULL&&msg->command==ASERVER_CMD_FILE_SEEK) {
     boolean timeout;
-    int alarm_handle=lives_alarm_set(LIVES_ACONNECT_TIMEOUT);
+    int alarm_handle=lives_alarm_set(LIVES_DEFAULT_TIMEOUT);
     while (!(timeout=lives_alarm_get(alarm_handle))&&pulse_get_msgq(pulsed)!=NULL) {
       sched_yield(); // wait for seek
     }
@@ -1265,7 +1274,7 @@ boolean pulse_audio_seek_frame(pulse_driver_t *pulsed, int frame) {
   // position will be adjusted to (floor) nearest sample
   int64_t seekstart;
   volatile aserver_message_t *pmsg;
-  int alarm_handle=lives_alarm_set(LIVES_ACONNECT_TIMEOUT);
+  int alarm_handle=lives_alarm_set(LIVES_DEFAULT_TIMEOUT);
   boolean timeout;
 
   if (alarm_handle==-1) {
@@ -1299,7 +1308,7 @@ int64_t pulse_audio_seek_bytes(pulse_driver_t *pulsed, int64_t bytes) {
   volatile aserver_message_t *pmsg;
 
   boolean timeout;
-  int alarm_handle=lives_alarm_set(LIVES_ACONNECT_TIMEOUT);
+  int alarm_handle=lives_alarm_set(LIVES_DEFAULT_TIMEOUT);
 
   int64_t seekstart;
 
@@ -1383,7 +1392,7 @@ void pulse_aud_pb_ready(int fileno) {
       boolean timeout;
       int alarm_handle;
 
-      if (tmpfilename!=NULL) lives_free(tmpfilename);
+      lives_freep((void **)&tmpfilename);
       mainw->pulsed->in_achans=sfile->achans;
       mainw->pulsed->in_asamps=sfile->asampsize;
       mainw->pulsed->in_arate=sfile->arate;
@@ -1396,7 +1405,7 @@ void pulse_aud_pb_ready(int fileno) {
         mainw->pulsed->reverse_endian=TRUE;
       else mainw->pulsed->reverse_endian=FALSE;
 
-      alarm_handle=lives_alarm_set(LIVES_ACONNECT_TIMEOUT);
+      alarm_handle=lives_alarm_set(LIVES_DEFAULT_TIMEOUT);
       while (!(timeout=lives_alarm_get(alarm_handle))&&pulse_get_msgq(mainw->pulsed)!=NULL) {
         sched_yield(); // wait for seek
       }
